@@ -22,10 +22,12 @@
 #include "config.h"
 
 #include "gcr-certificate.h"
+#include "gcr-certificate-extensions.h"
 #include "gcr-comparable.h"
 #include "gcr-icons.h"
 #include "gcr-internal.h"
 #include "gcr-oids.h"
+#include "gcr-subject-public-key.h"
 
 #include "egg/egg-asn1x.h"
 #include "egg/egg-asn1-defs.h"
@@ -177,97 +179,6 @@ certificate_info_load (GcrCertificate *cert)
 	return info;
 }
 
-static guint
-calculate_rsa_key_size (EggBytes *data)
-{
-	EggBytes *content;
-	guint key_size;
-	GNode *asn;
-
-	asn = egg_asn1x_create_and_decode (pk_asn1_tab, "RSAPublicKey", data);
-	g_return_val_if_fail (asn != NULL, 0);
-
-	content = egg_asn1x_get_raw_value (egg_asn1x_node (asn, "modulus", NULL));
-	if (!content)
-		g_return_val_if_reached (0);
-
-	egg_asn1x_destroy (asn);
-
-	/* Removes the complement */
-	key_size = (egg_bytes_get_size (content) / 2) * 2 * 8;
-
-	egg_bytes_unref (content);
-	return key_size;
-}
-
-static guint
-calculate_dsa_params_size (EggBytes *data)
-{
-	EggBytes *content;
-	gsize params_size;
-	GNode *asn;
-
-	asn = egg_asn1x_create_and_decode (pk_asn1_tab, "DSAParameters", data);
-	g_return_val_if_fail (asn != NULL, 0);
-
-	content = egg_asn1x_get_raw_value (egg_asn1x_node (asn, "p", NULL));
-	if (!content)
-		g_return_val_if_reached (0);
-
-	egg_asn1x_destroy (asn);
-
-	/* Removes the complement */
-	params_size = (egg_bytes_get_size (content) / 2) * 2 * 8;
-
-	egg_bytes_unref (content);
-	return params_size;
-}
-
-static guint
-calculate_key_size (GcrCertificateInfo *info)
-{
-	GNode *asn;
-	EggBytes *data;
-	guint key_size = 0, n_bits;
-	EggBytes *key;
-	GQuark oid;
-
-	data = egg_asn1x_get_raw_element (egg_asn1x_node (info->asn1, "tbsCertificate", "subjectPublicKeyInfo", NULL));
-	g_return_val_if_fail (data != NULL, 0);
-
-	asn = egg_asn1x_create_and_decode (pkix_asn1_tab, "SubjectPublicKeyInfo", data);
-	g_return_val_if_fail (asn != NULL, 0);
-
-	egg_bytes_unref (data);
-
-	/* Figure out the algorithm */
-	oid = egg_asn1x_get_oid_as_quark (egg_asn1x_node (asn, "algorithm", "algorithm", NULL));
-	g_return_val_if_fail (oid, 0);
-
-	/* RSA keys are stored in the main subjectPublicKey field */
-	if (oid == GCR_OID_PKIX1_RSA) {
-
-		/* A bit string so we cannot process in place */
-		key = egg_asn1x_get_bits_as_raw (egg_asn1x_node (asn, "subjectPublicKey", NULL), &n_bits);
-		g_return_val_if_fail (key != NULL, 0);
-		key_size = calculate_rsa_key_size (key);
-		egg_bytes_unref (key);
-
-	/* The DSA key size is discovered by the prime in params */
-	} else if (oid == GCR_OID_PKIX1_DSA) {
-		key = egg_asn1x_get_raw_element (egg_asn1x_node (asn, "algorithm", "parameters", NULL));
-		key_size = calculate_dsa_params_size (key);
-		egg_bytes_unref (key);
-
-	} else {
-		g_message ("unsupported key algorithm in certificate: %s", g_quark_to_string (oid));
-	}
-
-	egg_asn1x_destroy (asn);
-
-	return key_size;
-}
-
 static GChecksum*
 digest_certificate (GcrCertificate *self, GChecksumType type)
 {
@@ -295,7 +206,7 @@ calculate_markup (GcrCertificate *self)
 	gchar *markup;
 
 	g_object_get (self, "label", &label, NULL);
-	issuer = gcr_certificate_get_issuer_cn (self);
+	issuer = gcr_certificate_get_issuer_name (self);
 
 	if (issuer)
 		markup = g_markup_printf_escaped ("%s\n<small>Issued by: %s</small>", label, issuer);
@@ -466,6 +377,31 @@ gcr_certificate_get_der_data (GcrCertificate *self,
 	return GCR_CERTIFICATE_GET_INTERFACE (self)->get_der_data (self, n_data);
 }
 
+/**
+ * gcr_certificate_get_issuer_name:
+ * @self: a #GcrCertificate
+ *
+ * Get a name to represent the issuer of this certificate.
+ *
+ * This will try to lookup the common name, orianizational unit,
+ * organization in that order.
+ *
+ * Returns: the allocated issuer name, or NULL if no issuer name
+ */
+gchar *
+gcr_certificate_get_issuer_name (GcrCertificate *self)
+{
+	gchar *name;
+
+	name = gcr_certificate_get_issuer_part (self, "cn");
+	if (name == NULL)
+		name = gcr_certificate_get_issuer_part (self, "ou");
+	if (name == NULL)
+		name = gcr_certificate_get_issuer_part (self, "o");
+
+	return name;
+}
+
 /**
  * gcr_certificate_get_issuer_cn:
  * @self: a #GcrCertificate
@@ -524,7 +460,7 @@ _gcr_certificate_get_issuer_const (GcrCertificate *self)
 	info = certificate_info_load (self);
 	g_return_val_if_fail (info, NULL);
 
-	return egg_asn1x_get_raw_element (egg_asn1x_node (info->asn1, "tbsCertificate", "issuer", NULL));
+	return egg_asn1x_get_element_raw (egg_asn1x_node (info->asn1, "tbsCertificate", "issuer", NULL));
 }
 
 /**
@@ -638,6 +574,31 @@ gcr_certificate_get_subject_cn (GcrCertificate *self)
 	return gcr_certificate_get_subject_part (self, "cn");
 }
 
+/**
+ * gcr_certificate_get_subject_name:
+ * @self: a #GcrCertificate
+ *
+ * Get a name to represent the subject of this certificate.
+ *
+ * This will try to lookup the common name, orianizational unit,
+ * organization in that order.
+ *
+ * Returns: the allocated subject name, or NULL if no subject name
+ */
+gchar *
+gcr_certificate_get_subject_name (GcrCertificate *self)
+{
+	gchar *name;
+
+	name = gcr_certificate_get_subject_part (self, "cn");
+	if (name == NULL)
+		name = gcr_certificate_get_subject_part (self, "ou");
+	if (name == NULL)
+		name = gcr_certificate_get_subject_part (self, "o");
+
+	return name;
+}
+
 /**
  * gcr_certificate_get_subject_part:
  * @self: a #GcrCertificate
@@ -704,7 +665,7 @@ _gcr_certificate_get_subject_const (GcrCertificate *self)
 	info = certificate_info_load (self);
 	g_return_val_if_fail (info, NULL);
 
-	return egg_asn1x_get_raw_element (egg_asn1x_node (info->asn1, "tbsCertificate", "subject", NULL));
+	return egg_asn1x_get_element_raw (egg_asn1x_node (info->asn1, "tbsCertificate", "subject", NULL));
 }
 
 /**
@@ -815,14 +776,18 @@ guint
 gcr_certificate_get_key_size (GcrCertificate *self)
 {
 	GcrCertificateInfo *info;
+	GNode *subject_public_key;
 
 	g_return_val_if_fail (GCR_IS_CERTIFICATE (self), 0);
 
 	info = certificate_info_load (self);
 	g_return_val_if_fail (info, 0);
 
-	if (!info->key_size)
-		info->key_size = calculate_key_size (info);
+	if (!info->key_size) {
+		subject_public_key = egg_asn1x_node (info->asn1, "tbsCertificate",
+		                                     "subjectPublicKeyInfo", NULL);
+		info->key_size = _gcr_subject_public_key_calculate_size (subject_public_key);
+	}
 
 	return info->key_size;
 }
@@ -985,6 +950,42 @@ gcr_certificate_get_icon (GcrCertificate *self)
 	return g_themed_icon_new (GCR_ICON_CERTIFICATE);
 }
 
+/**
+ * gcr_certificate_get_basic_constraints:
+ * @self: the certificate
+ * @is_ca: (allow-none): location to place a %TRUE if is an authority
+ * @path_len: (allow-none): location to place the max path length
+ *
+ * Get the basic constraints for the certificate if present. If %FALSE is
+ * returned then no basic constraints are present and the @is_ca and
+ * @path_len arguments are not changed.
+ *
+ * Returns: whether basic constraints are present or not
+ */
+gboolean
+gcr_certificate_get_basic_constraints (GcrCertificate *self,
+                                       gboolean *is_ca,
+                                       gint *path_len)
+{
+	GcrCertificateInfo *info;
+	EggBytes *value;
+
+	g_return_val_if_fail (GCR_IS_CERTIFICATE (self), FALSE);
+
+	info = certificate_info_load (self);
+	g_return_val_if_fail (info, FALSE);
+
+	value = _gcr_certificate_extension_find (info->asn1, GCR_OID_BASIC_CONSTRAINTS, NULL);
+	if (!value)
+		return FALSE;
+
+	if (!_gcr_certificate_extension_basic_constraints (value, is_ca, path_len))
+		g_return_val_if_reached (FALSE);
+
+	egg_bytes_unref (value);
+	return TRUE;
+}
+
 /* -----------------------------------------------------------------------------
  * MIXIN
  */
@@ -1002,6 +1003,14 @@ gcr_certificate_get_icon (GcrCertificate *self)
  * </programlisting></informalexample>
  */
 
+/**
+ * gcr_certificate_mixin_emit_notify:
+ * @self: the #GcrCertificate
+ *
+ * Implementers of the #GcrCertificate mixin should call this function to notify
+ * when the certificate has changed to emit notifications on the various
+ * properties.
+ */
 void
 gcr_certificate_mixin_emit_notify (GcrCertificate *self)
 {
@@ -1112,10 +1121,10 @@ gcr_certificate_mixin_get_property (GObject *obj, guint prop_id,
 
 	switch (prop_id) {
 	case PROP_LABEL:
-		g_value_take_string (value, gcr_certificate_get_subject_cn (cert));
+		g_value_take_string (value, gcr_certificate_get_subject_name (cert));
 		break;
 	case PROP_SUBJECT:
-		g_value_take_string (value, gcr_certificate_get_subject_cn (cert));
+		g_value_take_string (value, gcr_certificate_get_subject_name (cert));
 		break;
 	case PROP_ICON:
 		g_value_set_object (value, gcr_certificate_get_icon (cert));
@@ -1127,7 +1136,7 @@ gcr_certificate_mixin_get_property (GObject *obj, guint prop_id,
 		g_value_take_string (value, calculate_markup (cert));
 		break;
 	case PROP_ISSUER:
-		g_value_take_string (value, gcr_certificate_get_issuer_cn (cert));
+		g_value_take_string (value, gcr_certificate_get_issuer_name (cert));
 		break;
 	case PROP_EXPIRY:
 		g_value_take_boxed (value, gcr_certificate_get_expiry_date (cert));
