@@ -35,24 +35,19 @@
 /**
  * SECTION:gcr-mock-prompter
  * @title: GcrMockPrompter
- * @short_description: XXX
+ * @short_description: a mock GcrSystemPrompter for testing
  *
- * XXXX
- */
-
-/**
- * GcrMockPrompter:
+ * A mock GcrSystemPrompter used for testing against.
  *
- * XXX
- */
-
-/**
- * GcrMockPrompterClass:
+ * Use gcr_mock_prompter_start() to start the mock prompter in another
+ * thread. The returned string is the dbus address of the mock prompter.
+ * You can pass this to gcr_system_prompt_open() as the prompter bus name.
  *
- * The class for #GcrMockPrompter.
+ * Use the gcr_mock_prompter_expect_confirm_ok() function and friends before
+ * prompting to verify that the prompts are displayed as expected, and to
+ * provide a response.
  */
 
-
 GType   _gcr_mock_prompt_get_type       (void) G_GNUC_CONST;
 #define GCR_TYPE_MOCK_PROMPT            (_gcr_mock_prompt_get_type ())
 #define GCR_MOCK_PROMPT(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GCR_TYPE_MOCK_PROMPT, GcrMockPrompt))
@@ -76,7 +71,9 @@ enum {
 	PROP_PASSWORD_STRENGTH,
 	PROP_CHOICE_LABEL,
 	PROP_CHOICE_CHOSEN,
-	PROP_CALLER_WINDOW
+	PROP_CALLER_WINDOW,
+	PROP_CONTINUE_LABEL,
+	PROP_CANCEL_LABEL,
 };
 
 struct _GcrMockPrompt {
@@ -195,6 +192,8 @@ _gcr_mock_prompt_init (GcrMockPrompt *self)
 	blank_string_property (self->properties, "warning");
 	blank_string_property (self->properties, "choice-label");
 	blank_string_property (self->properties, "caller-window");
+	blank_string_property (self->properties, "continue-label");
+	blank_string_property (self->properties, "cancel-label");
 
 	blank_boolean_property (self->properties, "choice-chosen");
 	blank_boolean_property (self->properties, "password-new");
@@ -220,6 +219,8 @@ _gcr_mock_prompt_set_property (GObject *obj,
 	case PROP_CHOICE_LABEL:
 	case PROP_CHOICE_CHOSEN:
 	case PROP_CALLER_WINDOW:
+	case PROP_CONTINUE_LABEL:
+	case PROP_CANCEL_LABEL:
 		param = g_new0 (GParameter, 1);
 		param->name = pspec->name;
 		g_value_init (&param->value, pspec->value_type);
@@ -252,6 +253,8 @@ _gcr_mock_prompt_get_property (GObject *obj,
 	case PROP_CHOICE_LABEL:
 	case PROP_CHOICE_CHOSEN:
 	case PROP_CALLER_WINDOW:
+	case PROP_CONTINUE_LABEL:
+	case PROP_CANCEL_LABEL:
 		param = g_hash_table_lookup (self->properties, pspec->name);
 		g_return_if_fail (param != NULL);
 		g_value_copy (&param->value, value);
@@ -388,6 +391,8 @@ _gcr_mock_prompt_class_init (GcrMockPromptClass *klass)
 	g_object_class_override_property (gobject_class, PROP_CHOICE_CHOSEN, "choice-chosen");
 	g_object_class_override_property (gobject_class, PROP_PASSWORD_NEW, "password-new");
 	g_object_class_override_property (gobject_class, PROP_PASSWORD_STRENGTH, "password-strength");
+	g_object_class_override_property (gobject_class, PROP_CONTINUE_LABEL, "continue-label");
+	g_object_class_override_property (gobject_class, PROP_CANCEL_LABEL, "cancel-label");
 }
 
 static gboolean
@@ -453,7 +458,7 @@ gcr_mock_prompt_confirm_finish (GcrPrompt *prompt,
 	                      gcr_mock_prompt_confirm_async), GCR_PROMPT_REPLY_CANCEL);
 
 	return g_simple_async_result_get_op_res_gboolean (G_SIMPLE_ASYNC_RESULT (result)) ?
-	               GCR_PROMPT_REPLY_OK : GCR_PROMPT_REPLY_CANCEL;
+	               GCR_PROMPT_REPLY_CONTINUE : GCR_PROMPT_REPLY_CANCEL;
 }
 
 static void
